/*************************************************************************************************

                           PCM3 8051 Test Firmware
--------------------------------------------------------------------------------------------------
	

	Project: 				8051_Test DA/M
	Filename: 				PCM3.c
	Project Filename:		PCM3.uv2	(Keil uVision Launch file)
	Revision:				1.01
	Created by:				GJO
	Created on:				Aug 16, 2014
	Last Modified on:		Sep 17, 2014
	Target Processor:		Atmel AT89LP51ED2 running at 16.0 MHz
    Compiler used:    		uVision2 IDE with C51 compiler or uVision4 C51 V9.51b (Silabs version)
	Include file(s):		at89lp52.H, at89c51rc2.H

	Dev. Hardware:			Mikroe 8051-Ready board with AT89C51ED2 MCU plugged in and 16MHz Clock.

	Programmer used:		ASIX PRESTO USB  via SPI port (CN5)

	FLIP settings:			Hardware Byte:	0x3b and X2 enabled
							Security Level: 0
		
	Memory Model:			functions>2K, program<64K

	Chips tested:			AT89C51RC2 		- OK	BL	FLIP - PRESTO can't be used
							AT89LP51RC2 	- OK	ISP with ASIX PRESTO: use external RAM disabled 
													    and quasi-directional port settings
							AT89LP51ED2 	- OK	same IS settings as AT89LP51RC2
							AT89LP6440 		- ?		Not working yet with PRESTO - Appears to be inactive

	History:
	-------------------------------------------------------------------------------------------------
	V1.00	14-08-16	GJO		Initial port of original PCM3 PIC DA/M program to 8051 from 2000
								- Serial COMM works (U1V, U1U, U1A ,U1XI, U1XO)
								- No EEPROM support yet, No ADC present. Will switch later
								  to AT89LP51RC2 device with built in EEPROM and ADC
								- Note: The Baud rate generator needs to be revised and a
								  different crystal should be used to get accurate faster
								  baud rates. Suggest to use the 11.0591 MHz crystal.

	V1.00	14-08-17	GJO		Status:
								- Watchdog works
								- XI/XO command works
								- Added TUIMER0 MODE 1 5ms ticktimer
								- Changed to X2 mode

	V1.00	14-08-19	GJO		Status:
								- Added I2C library and tested it with TMP101
								- Added EEPROM function
								- Added DS1307 RTC function

	V1.00	14.08.21	GJO		Status:
								- Added DA/M EEPROM Editor (PD,PE,PR,PY,PRn, Pn,m)
								- Works now on AT89LP51ED2 and RC2 chips with PRESTO settings modified.
								- Begin testing ADC code
	
	V1.01	14.09.10	GJO		Status:
								- Moved some code into functions
								- Added IO_Update
								- Removed initial TMP101 FLOAT calculation, saved 1452 bytes 

	V1.02	14.09.17	GJO		Status:
								- Added 4x20 HD44780 LCD via PCA9554 Driver board 
								- Added TPIC2810 LED driver
								- Added PCA9554 LED Indicator board support
								(Analogs not working yet - Work in Progress)

	Program Size:		code: 7186, data=23.7, xdata=219

	-----------------------------------------------------------------------------------------------

	Commands:	Un{command}		where n is unit address for RS485 systems
	CMD			Example			Decription
	-----------------------------------------------------------------------------------------------
	V			-				Version String
	A, An,m		U1A2,4 U1A8		Read analog voltages
	X							Port and regs commands
	 XI			XI2				Read Snapshot of (selected) Port IO
	 XIn,m		XI2,7			Read regs 2 through 7
	 XOn,m		X2,127			Write data to selected port and registers
	P							Manage EEPROM commands
	 PR							 Read all EEPROM registers in 16-bit signed format
	 PE							 Erase all EEPROM cells
	 PD							 Restore default data
	 PRn						 Read selected EEPROM 16-bit register
	 PY							 EEPROM Hex dump of cells
	 Pn,m						 Write s16 word to EEPROM (127 regs)
	U	Ret: #10				Send error (returns #10)
	C	Ret: 15:12:34 14-08-26	Read DS1307 RTC
	@hhmmssyymmdd				Set DS1307 RTC, ex: @183945140825<CR><LF>
	T	Ret: 24.76 C			Read TMP101 temperature sensor
	U1^	Ret: PCM3 RESET			Watchdog Reset of system
	H							Help Screen

	ANALOG CHANNELS:
	----------------
	A0							P0.0 - 0-5V
	A1							P0.1 - 0-5V
	A2							P0.2 - 0-5V
	A3							P0.3 - 0-5V
	A4							P0.4 - 0-5V
	A5							P0.5 - 0-5V
	A6							TMP101 temperature in X100 Fixed point format Celsius
	A7							Die Temperature (relative)
	A8							VDD / 8 voltage

	REGISTER MAP:
	---------------------------------------------------------------------------------------------
	X0			-	P0			MCU port 0
	X1			-	P1			MCU port 1
	X2			-	P2			MCU port 2
	X3			-	P3			MCU port 3
	
	External devices:												I2C address
		
	X5			-	TPI2810		LED Control (8 LEDS)				0xCE
	X6			-	PCA9554 LED Board 07014G-I2C-LPNL output reg	0x4A
	X7			-	PCA9554 CFG REG									
	X8			-	PCA9554 POL INP REG
	X9			-	PCA9554 Input register
	
	X10-15		-	General Purpose regs

	EXT 24C02 EEPROM REGISTERS:	(126 registers available)	DEFAULT:
	---------------------------------------------------------------------------------------------
	EEP0	-	Unit Address when used 						1
	.
	.
	.
	EEP127	-	16-bit Checksum storage

	MCU PIN ASSIGNMENTS:									PORT CFG
	---------------------------------------------------------------------------------------------
	P0.0-5	-	tentatively reserved for analog inputs		INPUT

	P0.6	-	I2C SCL										BI_DI
	P0.7	-	I2C SDA										BI-DI

	P1		-	Free

	P2.7	- 	Heartbeat LED								BI-DI
	P2.0	-	ADC ISR indicator (diagnostics only			BI_DI

	P3.0	-	RS232 RXD									ALT FUN
	P3.1	-	RS232 TXD									ALT FUN

	I2C DEVICES CURRENTLY SUPPORTED
	---------------------------------------------------------------------------------------------
	0xA8	AT24C02				EEPROM 256x8
	0xD1	DS1307				RTC
	0x90	TMP101				Temperature Sensor
	0x4E	PCA9554				LCD Interface
	0x4A	PCA9554				LED indicator board
	0x4D	PCA9554				LED board keypad section
	0x42	PCA9554				Relay Board (not yet)


*/
//**********************************************************************************************
// Include files
//**********************************************************************************************
#include <reg51atlprb2.h>
//**********************************************************************************************
// System library & functions
//**********************************************************************************************
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

//**********************************************************************************************
//Defines and constants
//**********************************************************************************************

#define c_NUM_OF_DEFAULT_VALUES 	256

#define MAXREGS 					16
#define	MAX_ANALOGS					8

//EEPROM Locations
#define UNIT_NUMBER_ADDRESS 		0x00
#define SN_LB_ADDRESS				0x0F
#define SN_HB_ADDRESS				0x10
#define MFG_DATE_DAY_ADDRESS		0x11
#define MFG_DATE_MTH_ADDRESS		0x12
#define MFG_DATE_YEAR_ADDRESS_LB 	0x13
#define MFG_DATE_YEAR_ADDRESS_HB 	0x14
#define ANALOG_RESOLUTION        	0x21

#define MAX_LENGTH   34   		//Maximum # of characters allowed for command.
#define BUF_LENGTH   35   		//Command buffer size
#define FOREVER      1

#define ACK 1
#define NOACK 0

#define PCA9554_INPUT_PORT			0
#define PCA9554_OUTPUT_PORT			1
#define PCA9554_POL_INV_PORT		2
#define PCA9554_CFG_REG_PORT		3

//***********************************************************************************************
//	CONFIGURATION SETTINGS
//***********************************************************************************************

#define F_OSC			( 16000000 )	// <--- ENTER CRYSTAL FREQUENCY IN HZ HERE
#define ISR_T1_RATE		( 200 ) 		// <--- ENTER ISR REPEAT FREQUENCY IN HZ


#define WATCHDOG_ON 					// comment this line out to turn off watchdog timer for debugging
#define STARTUP_MESSAGE_ENABLED			// Turn OFF for RS485 applications

// Calculate T1 16-bit reload value
#define T1M1PERIOD16 	( (F_OSC / 12 ) / ISR_T1_RATE ) * -1	//-6827
#define T1_HIGH 		(65536 + T1M1PERIOD16) / 256
#define T1_LOW 			(65536 + T1M1PERIOD16) - (T1_HIGH * 256)

#define LED_PERIOD					( 200 )	// For 1s with 5ms T1 ISR rate

#define IO_UPDATE_INTERVALL			( 5 )	// IO update rate in n * 5ms

#define RTC_UPDATE_INTERVALL		( 100 )	// IO update rate in n * 5ms

//**********************************************************************************************
//register definitions not part of standard 51 header file
//**********************************************************************************************

sbit FE 		= SCON^7;

sbit LED_P2_7   = P2^6;	// Heartbeat (LED Plug has reversed even/odd order pins)
sbit LED_P2_6   = P2^7;	// CMD processing
sbit LED_P2_2   = P2^2;
sbit LED_P2_0   = P2^0;

sbit ADC_INTLED	= P2^1;

//sbit AEDC		= IEN1^5;

sbit PIN_P1_1   = P1^1;


//------------------------------- I2C --------------------------------------

sbit SCL=P0^6;      //SCL Connected to P0.6
sbit SDA=P0^7;      //SDA Connected to P0.7


//*********************** Command string variables *************************************************
volatile unsigned char  char_in_buf = 0;                   	// Stores the number of characters in the RX ring-buffer
volatile unsigned char  buf_index = 0;                     	// Current index in the receive ring-buffer
volatile unsigned char command_buf[BUF_LENGTH]="";        	// Holds the commands sent from the host.
volatile unsigned char command[MAX_LENGTH]="";     			// Holds command being executed.
volatile unsigned char action[MAX_LENGTH]="";

//**********************************************************************************************
//Global Variables
//**********************************************************************************************

unsigned char unit = 1;
volatile unsigned int _ms5cnt=0;

volatile unsigned int _analogs[MAX_ANALOGS];
volatile unsigned char _ioregs[MAXREGS];

volatile unsigned char _TPIC2810_dbyte=0;

unsigned char _last_XREG=0;

bit OK_FLAG = 0;
bit IO_UPDATE_FLAG=0;
bit RTC_UPDATE_FLAG=0;

//**********************************************************************************************
// FUNCTION PROTOTYPES
//**********************************************************************************************

// ISR functions
void timer1(void);
void get_char(void);

// Callable functions:

// System functions:
void initialize_hardware(void);
void send_error(unsigned char Code);
void print_one_ad(unsigned char port);
void print_ad(unsigned char channel);
void find_token(unsigned char offset);
void perform_command(unsigned char offset);
void check_cmd_buffer(void);
void is_unit_called();
void read_cal_values();
void print_port(unsigned char port);
void set_port(unsigned char portsel, unsigned char value);
void update_IO(void);
void update_RTC(void);

void restart_wdg(void);

void delay_us(unsigned int us_count);
void delay_ms(unsigned int ms_count);
void delay_sec(unsigned char sec_count);

// I2C functions
void I2C_Clock(void);
void I2C_Start(void);
void I2C_Stop(void);
//unsigned char I2C_Write(unsigned char dat);
unsigned char I2C_Write(unsigned char dat, unsigned char mode);
unsigned char I2C_Read(unsigned char mode);
void I2C_Ack();
void I2C_NoAck();
unsigned char I2C_Write_Ack_poll(unsigned char dat);


// Sensor functions
void init_TMP101();
void write_TMP101();
unsigned int read_TMP101(unsigned char mode);


// External EEPROM support
unsigned char EEPROM_ReadByte(unsigned char eeprom_Address);
unsigned char read_eeprom(unsigned char reg_addr);
void EEPROM_WriteByte(unsigned char eeprom_Address, unsigned char eeprom_Data);
void write_signed_eeprom_word(unsigned int lb_add, signed int datain);
signed int read_signed_eeprom_word(unsigned int lb_add);
void EEPROM_ReadByte_sequential(unsigned char eeprom_Address, unsigned int numbytes);
void read_cal_values(void);
void write_cal_values(void);

// DS1307 function prototypes
char toBCD(unsigned char bin_val);
void set_time(void);
void read_time(void);
void DS1307_read_time(unsigned char select);
void DS1307_Clear_all_RAM(void);
void DS1307_REG_DUMP(void);
void edit_RTCRAM(void);
//unsigned char RTC_ReadByte(unsigned char Address);

// TPIC2810 functions
void set_Cmd_TPIC2810(unsigned char cmd_byte);
void TPIC2810(unsigned char dbyte);

// Command functions
void read_ADC(void);
void read_sensor(void);
void print_version(void);
void test_watchdog(void);
void edit_XREGS(void);
void Edit_EEPROM(void);

void wr_half(unsigned char LCD_data, unsigned char cmd_mode);
void wr_disp(unsigned char LCD_data, unsigned char  cmd_mode);
void put_I2C_LCD(unsigned char c);
void line(unsigned char which_line);
void clear_display(void);
void set_LCD_cur(unsigned char LCDrow, unsigned char LCDcol);
void init_HD44780_LCD(void);
void clear_line(unsigned char line);
void write_PCA9554_LCD(unsigned char dbyte, unsigned char cmd_byte, unsigned char slvadr);
void puts_I2C_LCD(unsigned char *c);

void write_PCA9554_port(unsigned char dbyte, unsigned char cmd_byte, unsigned char I2C_address);
unsigned char read_PCA9554_port(unsigned char I2C_address);

//**********************************************************************************************
//**************************** Interrupt Driven Functions **************************************
//**********************************************************************************************

//*************************************************************************************************
// get_char():
// Description: This interrupt driven function receives characters from the host until and places
//              the characters in a ring-buffer.  If the number of characters in buffer reach
//              the BUF_LENGTH then the Receiver is disabled allowing no more characters in
//              the buffer.
//*************************************************************************************************

void get_char(void) interrupt 4 using 1
{
	unsigned char c;

	if(0 != RI)
	{
		c = toupper(SBUF);
		RI=0;

		command_buf[buf_index] = c;         	// Place character in buffer
		char_in_buf++;                      	// Increment characters in buffer variable

		if(buf_index == (BUF_LENGTH - 1)){   	// Check if at end of buffer
			buf_index = 0;                   	// Start inserting characters at the start
		}else{
			buf_index++;                     	// Move to next location
		}
		// Protect from Buffer Overrun when full
		if(char_in_buf == BUF_LENGTH)
		{
			ES=0;
	   	}
	}
}

#define MAKE16(a,b) ((unsigned int) (a << 8) + (unsigned int) b)

/*-----------------------------------------------------------------------------------*
                         void timer1 (void) interrupt 3 using 1
 ------------------------------------------------------------------------------------*
 * I/P Arguments: none.
 * Return value : none

 * description :	This ISR provides a 5ms system tick timebase at a rate of 200 Hz
					It uses the TIMER1 in mode 1 16-bit re-load mode.

					The Timing in s = 1/[OSC}/12 * n

					N[T1RL] = T[ISR}(s) / (1/F[OSC]/12)
									or
					N[T1RL] = (F[OSC] / 12) / ISR[HZ]

					Example: 0.005 / (1/16384000/12) = 6826.666... or 6827 for
					0.005224 s or 199.990235 Hz with an error of 0.004883% or 4.22s/day

					Will require device with auto re-load, 89C51RC2 will not work properly
					without fine tuning load delays. Obtain the LP versions instead
-------------------------------------------------------------------------------------*/
void timer1 (void) interrupt 3 using 2
{
	static unsigned int oneSecondCount=0;
	static unsigned int IO_UPDATECount=0;
	static unsigned int RTC_UPDATECount=0;

//	static unsigned char ADC_channel=8;

	if( IO_UPDATE_INTERVALL <= ++IO_UPDATECount)
	{
		IO_UPDATE_FLAG = 1;
		IO_UPDATECount=0;
	}

	if( RTC_UPDATE_INTERVALL <= ++RTC_UPDATECount)
	{
		RTC_UPDATE_FLAG = 1;
		RTC_UPDATECount=0;
	}

	++_ms5cnt;

	if( LED_PERIOD <= ++oneSecondCount)
	{
		oneSecondCount=0;
		LED_P2_7 = 0;	// ON
	}

	if( (LED_PERIOD/8) == oneSecondCount)
	{
		LED_P2_7 = 1;	// OFF
	}
/*
	// Check GO/BSY Flag
	if(0 == (DADC & 0x40))
	{
		ADC_INTLED = 0;
		// Measure VDD/8 channel for now
		_analogs[7] = MAKE16(DADH,DADL);
		ADC_INTLED = 1;
	}

	// Start ADC conversion by setting GO/BSY falg
	DADC |= (1 << 6);
*/
}

/*-----------------------------------------------------------------------------------*
                         void ADC_ISR(void) interrupt 12 using 1 (0x0063)
 ------------------------------------------------------------------------------------*
 * I/P Arguments: none.
 * Return value : none

 * description :
-------------------------------------------------------------------------------------*/
void ADC_ISR(void) interrupt 12 using 1
{


//	ADC_INTLED = 0;
	// Measure VDD/8 channel for now
	_analogs[7] = MAKE16(DADH,DADL);
//	ADC_INTLED = 1;
}



//**********************************************************************************************
//********************************* Callable Functions *****************************************
//**********************************************************************************************

void check_cmd_buffer(void)
{
//	unsigned char i=0;
	static unsigned char  command_size = 0;		// Holds position in command array for next character.
	static unsigned char  command_index = 0;	// Position in Receive buffer where the current
                                          		// command is being taken.
	static bit command_overflow = 0;            // This flag is set when the command is larger than
                                            	// MAX_LENGTH.
	// Frame error detection bit test
	if(FE == 1)
	{
		FE=0;	// Clear frame error bit
		printf("\r\nFRAME ERR");
	}

	if(0 != char_in_buf)                              		//New character received
	{
		if(command_size < MAX_LENGTH)             			//Check command size we don't wan't it to
		{                                         			// overflow
			if(command_buf[command_index] != '\n')
			{
				if(command_buf[command_index] == '\r') 		//Check for end of command
				{
					command[command_size] = '\0';       	//Replace '\r' with NULL
			   		if(!command_overflow)               	//If the previous command was to long don't execute the rest of it
			   		{
						is_unit_called();               	//Check unit number and process command
			   		}
			   		command_size = 0;                   	//Set command size back to zero
			   		command_overflow = 0;           		//Clear command_overflow flag
				}
				else
				{
			   		command[command_size] = command_buf[command_index];
			 	  	command_size++;
				}
			}
			char_in_buf--;
			if(command_index == (BUF_LENGTH - 1))   	//Check if at end of buffer
			{
				command_index = 0;                   	//Start taking characters from the start
			}
			else
			{
				command_index++;                     	//Move to next location
			}
		}
		else
		{  //Command is greater than MAX_LENGTH
			command_size = 0;                       	//Set command size back to zero
			command_overflow = 1;
		}
	}

   	if(char_in_buf < BUF_LENGTH)
	{
		ES=1;					// re-enable RX interrupt
	}
}


//**********************************************************************************************
//find_token(int offset):
//    This function breaks out a string into tokens.
//    Tokens are predefined. The variable offset is
//    a pointer into the command string that tells
//    find_token where to start looking for tokens.
//**********************************************************************************************
void find_token(unsigned char offset)
{
	code unsigned char tokens[12]={'A','C','P','S','T','U','V','X','^','@','~',' ' };

	strncpy(action, command + offset, MAX_LENGTH);

	// Go through action until NULL is hit always skipping the first character
	for (offset = 1; offset <= strlen(action) + 1 ; offset++)
	{
		if( strchr(tokens, action[offset]))	// Find start of next token to find end of last token
	  	{
	    	action[offset] = '\0';
	     	break;
	  	}
	}
}



//**********************************************************************************************
//is_unit_called(): Find out if unit is even called.
//                  If it is, perform the command.
//**********************************************************************************************
void is_unit_called()
{
	// Make sure there is something in command before calling find_token();
	if(strlen(command))
	{
		// Make sure command has a U in the first location
		if('U'== command[0] )
		{
			find_token(0);       			// Call find_token with no offset
			if( (1 < strlen(action)) )        // Make sure a unit# is after U
			{
				if( unit == (atoi(action + 1) ) )
				{
			   		perform_command(strlen(action));
				}
			}
		}
	}
}							         	// unit will perform all commands received.




//**********************************************************************************************
//perform_command():
//**********************************************************************************************
void perform_command(unsigned char offset)
{
	unsigned char temp = 0;


//	LED_P2_6 = 0;

	printf("\n");				   // Start of Message character

	while ( '\0' != command[offset] )
	{
	   	find_token(offset);

		switch (command[offset])
		{
		case 'A':	// n,m
			read_ADC();
		break;
        case '@':
			set_time();
        break;
        case 'C':	// 1, 9
			edit_RTCRAM();
		break;
		case 'P':	// R, D, E, n
			Edit_EEPROM();
 		break;
	    case '^':
			test_watchdog();
		break;
		case 'T':
			read_TMP101(1);
		break;
		case 'U':
			send_error(10); return;
		case 'V':
			print_version();
		break;
		case 'X':	// I, O, n,m
			edit_XREGS();
		break;

		default:
		break;	// skip unusable characters
		} 
		++offset;
	} 

    if(OK_FLAG != 0)
    {
        printf( "OK");         // Acknowledge command
        OK_FLAG=0;
    }

	printf("\r\n");  			// Finish up output with a LF,CR

//	LED_P2_6 = 1;
}



void test_watchdog(void)
{
	unsigned int i=0;
	while(1)
	{
		printf("Watchdog Test %u \r", ++i); 
		delay_ms(100);
	} 
}



void print_version(void)
{
	printf("PCM51 V1.02, "); printf(__DATE__); 
	putchar(' ');	// 48 bytes
}




void read_ADC(void)
{
	unsigned char index;  		//Used in various for loops

	if (strlen(action) == 1)
	{
		for (index = '1'; index <= '8'; index++)
		{
			print_one_ad(index);
		}
	}
	else if (strlen(action) == 2)
	{
		print_one_ad(action[1]);
	}
	else if (action[2] == ',')
	{
		for (index = action[1]; index <= action[3]; index++)
			print_one_ad(index);
	}
}



void edit_RTCRAM(void)
{
	if (action[1] == '1'){
        DS1307_REG_DUMP();
    }else if (action[1] == '9'){
        DS1307_Clear_all_RAM();
    }else{
        DS1307_read_time(1);   // DS1307 on board RTC
    }
}



// watchdog restart sequence
void restart_wdg(void)
{
#ifdef WATCHDOG_ON
	WDTRST=0x1E;
	WDTRST=0xE1;
#endif
}

//**********************************************************************************************
// send_error(int Code)
//**********************************************************************************************
void send_error(unsigned char Code)
{
	switch(Code)
	{
		case  4: printf("#4\n\r"); break;
		case 10: printf("#10\n\r"); break;
		default: printf("#%u", (unsigned int) Code); break;
	}
}

//**********************************************************************************************
// print_one_ad(int port): This reads the value of one A/D and prints it to the RS-232 port.
//**********************************************************************************************
void print_one_ad(unsigned char port)
{
	port = port - 48;	// Convert ASCII number to index

	if( (0 < port) && (9 > port) )
	{
		printf("%d ", (signed int) _analogs[port - 1]);
	}
}


//**********************************************************************************************
// set_port: This sets the specified output port.
//**********************************************************************************************
void set_port(unsigned char portsel, unsigned char value)
{
	switch(portsel)
	{
		case 0:    P0 = ~value;
        break;
		case 1:    P1 = ~value;
        break;
		case 2:    P2 = ~value;
		break;
		case 3:    P3 = ~value;
		break;
		default:
		break;
	}
}


void update_IO(void)
{
	unsigned char temp=0;

	if(0 == IO_UPDATE_FLAG) return;

	IO_UPDATE_FLAG = 0;

	temp = _last_XREG;

	if(4 > temp )
	{
		set_port( temp, _ioregs[temp]);	
	}
	if(5 == temp)
	{
		TPIC2810(_ioregs[temp]); 	
	}
	if(6 == temp)
	{
		// Invert data 0=LED=ON
		write_PCA9554_port(~_ioregs[temp], PCA9554_OUTPUT_PORT, 0x4A);	// LED indicator board
	}
	if(7 == temp)
	{
		write_PCA9554_port(_ioregs[temp], PCA9554_CFG_REG_PORT, 0x4A);
	}
	if(8 == temp)
	{
		write_PCA9554_port(_ioregs[temp], PCA9554_POL_INV_PORT, 0x4A);
	}
	if(10 == temp)
	{
		write_PCA9554_port(_ioregs[temp], PCA9554_POL_INV_PORT, 0x42);	// Relay board
	}

	_last_XREG	= 999;	// Reset last used XREG value	
}


void update_RTC(void)
{

	if(!RTC_UPDATE_FLAG) return;

	RTC_UPDATE_FLAG=0;


	DS1307_read_time(0);
	read_TMP101(0);

}

//**********************************************************************************************
// print_port(int port): This reads the value of one I/O port and prints it to the RS-232 port.
//**********************************************************************************************
void print_port(unsigned char port)
{
	unsigned char portval = 0;

	switch(port)
	{
	case 0: portval = P0;
        break;
	case 1: portval = P1;
        break;
	case 2: portval = P2;
        break;
	case 3: portval = P3;
        break;
	case 9:	portval = read_PCA9554_port(0x4A);
		break;	
	default : portval = _ioregs[port];
	}
	printf("%02x ", (unsigned int) portval );
}


/******************************************************************************
* Function:    	edit_XREGS(void)
* Description: 	This function allows to change and read the PORTS 0-3 and _ioregs[]
*              	locations.
*
* Commands:    	XI	reads all XREGs
*				XIn,m	where n is starting reg number and m is ending reg number 
*				XOn,m	where n is reg number and m is a 8-bit value 			
*
* Note:			Be careful changing port bits as some may be changed elsewhere
*				such as the heartbeat LED and cmd ack LED right now
*
* Hardware:    none
* Requires:    none
* Parameters:  none
* Return Val:  None
* Calls:       None
* Constants:   None
* Flags:       OK_FLAG
* Interrupts:  None
*******************************************************************************/
void edit_XREGS(void)	// 439 bytes
{

	unsigned char temp = 0;
	unsigned char portvalue;   	//Holds I/O port state
	unsigned char portnum;     	//Holds I/O port being modified or read
	unsigned char index;  		//Used in various for loops
	unsigned char start_index;  //
	unsigned char end_index;  	//

    unsigned char buff[16];
    unsigned char span;
    unsigned char* pos; 

			switch(action[1])
			{
			case 'I':
			if (strlen(action) == 2)
			{
				for (index = 0; index < MAXREGS; index++)
				{
					print_port(index);
				}
			}
			else
			{

				pos = strchr(action , ',');                     // Look for comma in cmd string
				if (pos)                                        // If found then extract data
				{
					span = pos - (action + 2);                  // Get len of regindex part of string
					strncpy (buff, action + 2, span);           // Extract  reg num between ..Inn,
					start_index = atoi(buff);
					end_index = atoi(pos + 1);
					for (index = start_index; index <= end_index; index++)
					{
						print_port(index);
					}
				}
				else if (strlen(action) >= 3)
				{
					index = atoi(action + 2);
					if(index < MAXREGS)
					{
						print_port(index);
					}
					else
					{
						send_error(15);
					}
				}
				else
				{
					send_error(14);
				}
			}
			break;

			case 'O':
				if(strlen(action) > 4)
				{
					pos = strchr(action , ',');                     // Look for comma in cmd string
					if (pos)                                        // If found then extract data
					{
						span = pos - (action + 2);                  // Get len of regindex part of string
						strncpy (buff, action + 2, span);           // Extract  reg num between ..Inn,
						buff[span]=0;                       		// Terminate string with null
						portnum = atoi(buff);
						portvalue = atoi(pos+1);
						if(portnum >= MAXREGS)
						{
							send_error(15);
						}
						else
						{
							_ioregs[portnum] = portvalue;
							_last_XREG = portnum;
							OK_FLAG = 1;
							IO_UPDATE_FLAG = 1;
						}
					}
					else
					{
						send_error(14);
					}
				}
            break;
			default:
				send_error(13);
			}

}



/******************************************************************************
* Function:    Edit_EEPROM
* Description: This function allows to change and read the external I2C EEEPROM
*                locations. It emulates 16 bit word size.
*
* Commands:    PD - Loads EEPROM with default values
*              PRn - Reads back EEPROM value, 16 bit, valid n = 0 to 13
*              Pn,value - Allows to write new value to EEPROM in 16 bit mode
* Hardware:    External I2C data EEPROM
* Requires:    AT24C0x connected to I2C bus
* Parameters:  none
* Return Val:  None
* Calls:       None
* Constants:   None
* Flags:       OK_FLAG
* Interrupts:  None
*******************************************************************************/
//
void Edit_EEPROM(void)
{
    unsigned int EEPROM_Address;
	unsigned int k;
    unsigned char span;
    unsigned char i, j=0;
    unsigned char buff[8]="";
    unsigned char* pos;
    signed int EEPROM_ldata;

    if (strlen(action) == 2)
    {
        if (action[1] == 'D')
        {

            EEPROM_WriteByte(0xFF, 0xFF);           // Forces Read Cal Values to load defaults.
			write_cal_values();                     // Write EEPROM defaults

        }

        if (action[1] == 'E')
        {
            printf("\r\nERASING EEPROM");
			for (k = 0; k < (c_NUM_OF_DEFAULT_VALUES); k++)
            {
				EEPROM_WriteByte(k , 0xFF);
				restart_wdg();
            }
			printf("\r\nDONE ");

        }

        if (action[1] == 'Y')
			EEPROM_ReadByte_sequential(0, 256);


		if (action[1] == 'R')
        {
            for (i = 0; i < (c_NUM_OF_DEFAULT_VALUES / 2); i++)
            {
				printf("%d ", (signed int) read_signed_eeprom_word(i * 2));
                if (8 <= ++j )
                {
                    putchar('\r');putchar('\n');
                    j = 0;
                }
				restart_wdg();
            }
        }
    }
    else
    {                                                 // For more then 2 characters found
        if(action[1] == 'R')
        {
            printf("\nEEW");
            if (strlen(action) > 2)
                if (atol(action + 2) <= 1023)
                {
                    EEPROM_Address = atol(action + 2) << 1;      // multiply Address by 2 using left shift
                    EEPROM_ldata = read_signed_eeprom_word(EEPROM_Address);
                    printf( "[%u] = %d [%04X]", (unsigned int) EEPROM_Address >> 1,
									(signed int) EEPROM_ldata, (unsigned int) EEPROM_ldata);
                }
        }
        else
        {                                              // EEPROM Write command detected by default
            if (strlen(action + 1) > 2)
            {                                           // Get length of cmd string beginning after P
                pos = strchr(action , ',');             // Look for comma in cmd string
                if (pos)
                {                                       // If found then extract address and data
                    span = pos - (action + 1);          // Get len of address string
                    strncpy (buff, action + 1, span);   // Extract EEPROM address between ..Pnn,
                    buff[span]='\0';						// Null terminate buffer
					EEPROM_Address = atol(buff);  //
                    EEPROM_ldata = atol (pos + 1);
					printf("\nUPD[%u] = %d ", EEPROM_Address, EEPROM_ldata );
                    write_signed_eeprom_word(EEPROM_Address << 1, EEPROM_ldata);
                    read_cal_values();
                }else{                                  // No comma found, time to get out.
                    send_error(16);
                    return;
                }
            }else{
                send_error(64);
            }
        }
    }
}


//**********************************************************************************************
// read_signed_eeprom_word():
//**********************************************************************************************
signed int read_signed_eeprom_word(unsigned int lb_add)
{
	unsigned char lb_read, hb_read;

	lb_read = EEPROM_ReadByte(lb_add);
	hb_read = EEPROM_ReadByte(lb_add + 1);

	return ( ( (signed int) hb_read << 8) | (signed int) lb_read);
}



//**********************************************************************************************
// write_signed_eeprom_word():
//**********************************************************************************************
void write_signed_eeprom_word(unsigned int lb_add, signed int datain)
{
	signed int ldata;
	ldata = datain;

	EEPROM_WriteByte(lb_add, (unsigned char) ldata);
	EEPROM_WriteByte(lb_add + 1, (unsigned char)(ldata >> 8));
}



//**********************************************************************************************
// read_cal_values():
//**********************************************************************************************
void read_cal_values(void)
{

}



//**********************************************************************************************
// read_cal_values():
//**********************************************************************************************
void write_cal_values(void)
{

}




/*----------------------------------------------------------------------------------
                      8051 delay library  with the crystal frequency 11.0592MHz
Filename: delay.c
Controller: P89V51RD2/89c51(8051 family)
Oscillator: 11.0592 MHz
Author: XploreLabz
website: www.xplorelabz.com
 ----------------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------------
                         void delay_us(unsigned int n)
 ----------------------------------------------------------------------------------
 * I/P Arguments: unsigned int.
 * Return value : none

 * description : This function is used generate delay in us.
         It generates an approximate delay of 10us for each count,
         if 5000 is passed as the argument then it generates a delay of apprx 50ms.

-----------------------------------------------------------------------------------*/
void delay_us(unsigned int us_count)
{
    while(us_count!=0)
    {
         us_count--;
    }
}





/*----------------------------------------------------------------------------------
                         void delay_ms(unsigned int n)
 -----------------------------------------------------------------------------------
 * I/P Arguments: unsigned int.
 * Return value : none

 * description:  This function is used generate delay in ms.
     It genarates a approximate delay of 1ms for each count,
     if 1000 is passed as the argument then it generates delay of apprx 1000ms(1sec)
-----------------------------------------------------------------------------------*/
void delay_ms(unsigned int ms_count)
{
	while(ms_count!=0)
	{
		delay_us(112);   //delay_us is called to generate 1ms delay
		ms_count--;
	}
}



 /*----------------------------------------------------------------------------------
                        void delay_sec(unsigned char sec_count)
  -----------------------------------------------------------------------------------
  * I/P Arguments: unsigned char.
  * Return value    : none

  * description:
      This function is used generate delay in sec .
      It genarates a approximate delay of 1sec for each count,
      if 10 is passed as the argument then it generates delay of apprx 10sec

    note: A max of 255 sec delay can be generated using this function.
 -----------------------------------------------------------------------------------*/
void delay_sec(unsigned char sec_count)
 {
     while(sec_count!=0)
     {
         delay_ms(1000);    //delay_ms is called to generate 1sec delay
         sec_count--;
     }
 }
/*-------------------------------------------------*/

/*---------------------------------------------------------------------------------*
                                 8051 I2C  library
Filename: I2C.c
Controller: P89V51RD2/89c51(8051 family)
Oscillator: 11.0592 MHz
Author: XploreLabz
website: www.xplorelabz.com

Note:
1.The SDA and SCL lines are connected to P0.7 and P0.6
2.The code can be modified to connect the
  SDA and SCL to any of the PORTs by changing the "sbit".
 ----------------------------------------------------------------------------------*/




/*---------------------------------------------------------------------------------*
                         void I2C_Clock()
 ----------------------------------------------------------------------------------*
 * I/P Arguments: none.
 * Return value : none

 * description  :This function is used to generate a clock pulse on SCL line.
-----------------------------------------------------------------------------------*/
void I2C_Clock(void)
{
    delay_us(0);
    SCL = 1;        // Wait for Some time and Pull the SCL line High

    delay_us(0);        // Wait for Some time
    SCL = 0;        // Pull back the SCL line low to Generate a clock pulse
}



/*---------------------------------------------------------------------------------*
                         void I2C_Start()
 ----------------------------------------------------------------------------------*
 * I/P Arguments: none.
 * Return value : none
 * description  :This function is used to generate I2C Start Condition.
                 Start Condition: SDA goes low when SCL is High.

-----------------------------------------------------------------------------------*/
void I2C_Start()
{
    SCL = 0;        // Pull SCL low
    SDA = 1;        // Pull SDA High
    delay_us(1);

    SCL = 1;        //Pull SCL high
    delay_us(1);

    SDA = 0;        //Now Pull SDA LOW, to generate the Start Condition
    delay_us(1);

    SCL = 0;        //Finally Clear the SCL to complete the cycle
}



/*-----------------------------------------------------------------------------------
                         void I2C_Stop()
 ------------------------------------------------------------------------------------
 * I/P Arguments: none.
 * Return value : none
 * description  :This function is used to generate I2C Stop Condition.
                 Stop Condition: SDA goes High when SCL is High.

------------------------------------------------------------------------------------*/

void I2C_Stop(void)
{
    SCL = 0;            // Pull SCL low
    delay_us(1);

    SDA = 0;            // Pull SDA  low
    delay_us(1);

    SCL = 1;            // Pull SCL High
    delay_us(1);

    SDA = 1;            // Now Pull SDA High, to generate the Stop Condition
}


/*---------------------------------------------------------------------------------*
                         void I2C_Write(unsigned char dat)
 ----------------------------------------------------------------------------------*
 * I/P Arguments: unsigned char-->8bit data to be sent.
 * Return value : none

 * description  :This function is used to send a byte on SDA line using I2C protocol
                 8bit data is sent bit-by-bit on each clock cycle.
                 MSB(bit) is sent first and LSB(bit) is sent at last.
                 Data is sent when SCL is low.

-----------------------------------------------------------------------------------*/
unsigned char I2C_Write(unsigned char dat, unsigned char mode)
{
    unsigned char i;

	for(i=0;i<8;i++)         // loop 8 times to send 1-byte of data
	{
	    SDA = dat & 0x80;    // Send Bit by Bit on SDA line
	    I2C_Clock();         // Generate Clock at SCL
	    dat = dat<<1;
	}
	SDA = 1;                 // Set SDA at last
	
	if(0 == mode)
	{
	    SDA = 1;        //Pull SDA high to indicate Negative/NO ACK
	    I2C_Clock();     // Generate the Clock
	    SCL = 1;        // Set SCL */
	}else{
	    SDA = 0;        //Pull SDA low to indicate Positive ACK
	    I2C_Clock();    //Generate the Clock
	    SDA = 1;        // Pull SDA back to High(IDLE state)
	}


	return 1;
}

/*-----------------------------------------------------------------------------------*
                         unsigned char I2C_Read()
 ------------------------------------------------------------------------------------*
 * I/P Arguments: none.
 * Return value : Unsigned char(received byte)

 * description :This fun is used to receive a byte on SDA line using I2C protocol.
               8bit data is received bit-by-bit each clock and finally packed into Byte.
               MSB(bit) is received first and LSB(bit) is received at last.

-------------------------------------------------------------------------------------*/
unsigned char I2C_Read(unsigned char mode)
{
    unsigned char i, dat=0x00;
	SDA=1;               //Make SDA as I/P

	for(i=0;i<8;i++)        // loop 8times to read 1-byte of data
	{
		delay_us(1);
		SCL = 1;            // Pull SCL High

		delay_us(1);

		dat = dat<<1;       //dat is Shifted each time and
		dat = dat | SDA;    //ORed with the received bit to pack into byte

		SCL = 0;            // Clear SCL to complete the Clock
	}
   	               
	if(0 == mode)
	{
	    SDA = 1;        //Pull SDA high to indicate Negative/NO ACK
	    I2C_Clock();    // Generate the Clock
	    SCL = 1;        // Set SCL */
	}else{
	    SDA = 0;        //Pull SDA low to indicate Positive ACK
	    I2C_Clock();    //Generate the Clock
	    SDA = 1;        // Pull SDA back to High(IDLE state)
	}
	return dat;			// Finally return the received Byte*
}



/*---------------------------------------------------------------------------------*
                         void I2C_Ack()
 ----------------------------------------------------------------------------------*
 * I/P Arguments: none.
 * Return value : none

 * description  :This function is used to generate a the Positive ACK
                 pulse on SDA after receiving a byte.
-----------------------------------------------------------------------------------*/
/*
void I2C_Ack()		// 3 bytes
{
    SDA = 0;        // Pull SDA low to indicate Positive ACK
    I2C_Clock();    // Generate the Clock
    SDA = 1;        // Pull SDA back to High(IDLE state)
}
*/




/*---------------------------------------------------------------------------------*
                         void I2C_NoAck()
 ----------------------------------------------------------------------------------*
 * I/P Arguments: none.
 * Return value : none

 * description  :This function is used to generate a the Negative/NO ACK
                 pulse on SDA after receiving all bytes.
-----------------------------------------------------------------------------------*/
/*
void I2C_NoAck()
{
    SDA = 1;        // Pull SDA high to indicate Negative/NO ACK
    I2C_Clock();    // Generate the Clock
    SCL = 1;        // Set SCL 
}
*/


/*****************************************************************************
*
*	LIBRARY CATALOG REFERENCE:	TMP101-I2C-Temperature sensor
*
*	NAME:		TMP101 Thermometer LIBRARY MODULE
*
*	Purpose:	Reads the data from a TMP101 Thermometer chip
*
*	VER:		1.00
*
*	History:
*
*   Ver     Date        Who     Description
*   ---------------------------------------------------------------------
*   1.00    07.01.09    GJO     Wrote TMP101 library module
*
*
*	Usage:	TMP101_data = read_TMP101();
*
*	Setup:	n/a
*
*
*	Hardware Interface Requirements:
*
*		SCL PIN 	= P0.6
*		SDA PIN  	= P0.7
*
*	Return Success:		12-bit signed value
*	Return Failure:		TBD
*
*	Fault Detection:	If no response, then timeout occurs
*
*	Resources:	TMP101 data sheet
*				Uses XploreLabz 8051 I2C library
*
******************************************************************************/




// TMP101 read and write addresses
#define TMP101_WRITE 0x90
#define TMP101_READ 0x91

//void init_TMP101();
//void write_TMP101();



void write_TMP101()
{
	I2C_Start();
   	I2C_Write(TMP101_WRITE, ACK); // Enable Write mode
   	I2C_Write(0x01, ACK);   // Configuration register selected
   	I2C_Write(0x60, ACK);   // 12 bit continuous mode selected
   	I2C_Stop();
}



unsigned int read_TMP101(unsigned char mode)
{
	unsigned int temp, tenth;
	unsigned char MSB_data, LSB_data;
	bit flag=0;
	unsigned char buf[10];


    I2C_Start();
    I2C_Write(TMP101_WRITE, ACK);
    I2C_Write(0, ACK);      			// Configuration register = read temperature with 14-bit resolution
    I2C_Start();
    I2C_Write(TMP101_READ, ACK);     // Enable read mode
	MSB_data = I2C_Read(ACK);
	LSB_data = I2C_Read(ACK);
    I2C_Stop();

	temp = (( (unsigned int) MSB_data * 256) + (unsigned int) LSB_data );

	// Print temperature in DEGC with 0.1 DEGC resolution
	// The following code avoids the use of signed or float arithmitic
	// Saves 1452 bytes of FLASH over floats !!! - Uses 146 bytes instead of 1452

	if(temp < 32768) 					// Check sign
		flag=1;							// Temperature is positive
	else
		temp =  65535 - temp;			// Temp is negative, but convert to positive number
	
	tenth = temp % 256;					// Calculate fractional part of temperature
	temp = temp / 256;					// Calculate integer part
	tenth = ((tenth + 5) * 10)  / 256; 	// Calculate tenth deg C

	if(flag)	
		sprintf(buf, "%d.%u", temp, tenth);	// Temperature is positive
	else
		sprintf(buf, "-%d.%u", temp, tenth);	// Temperature is negative

	if(0 == mode){	
		set_LCD_cur(1, 0);
		puts_I2C_LCD(buf);
		put_I2C_LCD(0xdf);
		put_I2C_LCD('C');
	}else{
		puts(buf);
		putchar(' ');
	}

	return temp;
}



// TPIC2810 LED BOARD DRIVER

#define TPIC2810_WRITE 0xCE

// Write new value
void TPIC2810(unsigned char dbyte)
{
   I2C_Start();
   I2C_Write(TPIC2810_WRITE, ACK);		// Enable Write mode
   I2C_Write(0x44, ACK); 				// Use immediate transfer mode
   I2C_Write(dbyte, ACK);			    // Write Led Pattern
   I2C_Stop();
}

/*
// Write command byte to TPIC2810
void set_Cmd_TPIC2810(unsigned char cmd_byte)
{
   I2C_Start();
   I2C_Write(TPIC2810_WRITE, ACK); 		// Enable Write mode
   I2C_Write(cmd_byte, ACK);   			// Command
   I2C_Stop();
}
*/




//************************************************************************************************

unsigned char I2C_Write_Ack_poll(unsigned char dat)
{
    unsigned char i, status;

	for(i=0;i<8;i++)         // loop 8 times to send 1-byte of data
	{
	    SDA = dat & 0x80;    // Send Bit by Bit on SDA line
	    I2C_Clock();         // Generate Clock at SCL
	    dat = dat<<1;
	}
	SDA = 1;                 // Set SDA at last

	delay_us(1);
    SCL = 1;        // Wait for Some time and Pull the SCL line High

    delay_us(1);        // Wait for Some time
	status = SDA;
    SCL = 0;        // Pull back the SCL line low to Generate a clock pulse

	return status;
}


/* EEPROM_ID is fixed and should not be changed,
   Refer At2404/02 DataSheet for more info*/
#define EEPROM_ID 0xA8

// EEPROM USED: 24c02 256b (2Kb) 0xA0-0xA8, page size = 16B

/*---------------------------------------------------------------------------------------
  void EEPROM_WriteByte(unsigned char eeprom_Address, unsigned char eeprom_Data)
 ----------------------------------------------------------------------------------------
 * I/P Arguments: char,char-->eeprom_address at which eeprom_data is to be written.
 * Return value : none

 * description:This function is used to write the data at specified EEPROM_address..
               At2404 ic is enabled by sending its ID on the i2c bus.
               After selecting At2404 ic,select the address where the data is to written
               Write the Data at selected EppromAddress
               Stop the I2c communication.
----------------------------------------------------------------------------------------*/
void EEPROM_WriteByte(unsigned char eeprom_Address, unsigned char eeprom_Data)
{
	//printf("\r\nWEP=%u, %x ", (unsigned int) eeprom_Address, (unsigned int) eeprom_Data );

	I2C_Start();               	// Start i2c communication
    I2C_Write(EEPROM_ID, ACK);      	// connect to AT2404 by sending its ID on I2c Bus
    I2C_Write(eeprom_Address, ACK); 	// Select the Specified EEPROM address of AT2404
    I2C_Write(eeprom_Data, ACK);    	// Write the data at specified address
    I2C_Stop();                	// Stop i2c communication after Writing the data

	// Wait for write operation to finish by polling the EEPROM SDA line
	I2C_Start();
	while(1 == I2C_Write_Ack_poll(EEPROM_ID))
	{
		I2C_Start();
		delay_ms(1);
	}
}




/*-----------------------------------------------------------------------------------------
               unsigned char EEPROM_ReadByte(unsigned char eeprom_Address)
 ------------------------------------------------------------------------------------------
 * I/P Arguments: char-->eeprom_address from where eeprom_data is to be read.
 * Return value : char-->data read from Eeprom.

 * description:
           This function is used to read the data from specified EEPROM_address.
           At2404 ic is enabled by sending its ID on the i2c bus.
           After selecting At2404 ic,select the address from where the data is to read
           Read the Data from selected EppromAddress
           Stop the I2c communication.
           Return the Data read from Eeprom
-----------------------------------------------------------------------------------------*/
unsigned char EEPROM_ReadByte(unsigned char eeprom_Address)
{
  unsigned char eeprom_Data;

    I2C_Start();               			// Start i2c communication
    I2C_Write(EEPROM_ID, ACK);      	// connect to AT2404(write) by sending its ID on I2c Bus
    I2C_Write(eeprom_Address, ACK); 	// Select the Specified EEPROM address of AT2404

    I2C_Start();               			// Start i2c communication
    I2C_Write(EEPROM_ID | 0x01, ACK);	// connect to AT2404(read) by sending its ID on I2c Bus
    eeprom_Data = I2C_Read(NOACK);  	// Read the data from specified address
    I2C_Stop();                			// Stop i2c communication after Reading the data
    delay_us(10);
  return eeprom_Data;          			// Return the Read data

}



/*-----------------------------------------------------------------------------------------
               unsigned char EEPROM_ReadByte_sequential(unsigned char eeprom_Address)
 ------------------------------------------------------------------------------------------
 * I/P Arguments: char-->eeprom_address from where eeprom_data is to be read.
 *					char-->numbytes read from EEPROM
 * Return value : char-->data read from Eeprom.

 * description:
           This function is used to read the data from specified EEPROM_address.
           At2404 ic is enabled by sending its ID on the i2c bus.
           After selecting At2404 ic,select the address from where the data is to read
           Read the Data from selected EppromAddress
           Stop the I2c communication.
           Return the Data read from Eeprom
-----------------------------------------------------------------------------------------*/
void EEPROM_ReadByte_sequential(unsigned char eeprom_Address, unsigned int numbytes)
{
	unsigned char bl;
	unsigned int i;

	I2C_Start();               			// Start i2c communication
	I2C_Write(EEPROM_ID, ACK);      	// connect to AT2404(write) by sending its ID on I2c Bus
	I2C_Write(eeprom_Address, ACK); 	// Select the Specified EEPROM address of AT2404

	I2C_Start();               			// Start i2c communication
	I2C_Write(EEPROM_ID | 0x01, ACK);	// connect to AT2404(read) by sending its ID on I2c Bus
	for(i=0; i < numbytes - 1; i++)
	{
		bl = I2C_Read(ACK);				// Read the data from specified address
		printf("%02x ", (unsigned int) bl );
		restart_wdg();
	}
	bl = I2C_Read(NOACK);				// Read the data from specified address
	I2C_Stop();                // Stop i2c communication after Reading the data
	printf("%02x ", (unsigned int) bl );
	putchar('\r'); putchar('\n');
}


// DS1307 control bytes
#define WRITE_CNTRL 0xd0
#define READ_CNTRL 0xd1

// DS1307 register offsets
#define SECONDS_REG 0
#define MINUTES_REG 1
#define HOURS_REG 2
#define DAY_OF_WEEK_REG 3
#define DATE_REG 4
#define MONTH_REG 5
#define YEAR_REG 6
#define CONTROL_REG 7
#define DATE_TIME_BYTE_COUNT 8
#define DS1307_NVRAM_START_ADDR 8
#define CONTROL_INIT_VAL 			0x90

unsigned char ds1307_regs[DATE_TIME_BYTE_COUNT];

// Adapted from original Source from: http://www.rentron.com/CCS_C/SERLED_C.htm
//------------------------------------------------------------------
//
// Set Time use: U1@hhmmssyymmdd
//               U1@211203060523 ENTER
//
//



// set time & date on power-up
void set_time(void)
{

	unsigned char code reg_translate[8] = { 2, 1, 0, 6, 5, 4, 3, 0 };
    unsigned char regs[8];                          // Holds packed BCD register values
    unsigned char RTC_length;                       // Length of input string
    unsigned char fields = 0;                       // keeps track of the number of digit pairs
    unsigned char i;
    unsigned char temp, temp2, bcd;
    unsigned char ctemp[2];                         // temporary holds data nibble
    ctemp[1] = '\0';                                // add null char to end so atol can work

    memset(regs, 0, 8);

    RTC_length = strlen(action);                    // Get length of input string

    if (RTC_length > 1 ){

        for (i=1 ; i < RTC_length ; i += 2)     // Scan along time date string to pick up pairs
		{
            ctemp[0] = action[ i ];             // Store first digit
            temp = atoi(ctemp);
            ctemp[0] = action[ i + 1 ];         // Store second digit
            temp2 = atoi(ctemp);
            bcd = temp << 4;                        // shift 4 bits left to make room for two BCD digits (*16)
            regs[fields] = bcd + temp2;      	// bcd = packed BCD value ready to send to DS1306

            fields++;                           // Increment DS1306 register pointer
        }

        ds1307_regs[reg_translate[SECONDS_REG]] &= 0x7f;
        ds1307_regs[reg_translate[HOURS_REG]] &= 0x3f;


        // write 7 bytes of BCD data to ds1307
        I2C_Start();
        I2C_Write(WRITE_CNTRL, ACK);  // DS1307 Address
		//start at seconds register (Base Address of data registers)
        I2C_Write(SECONDS_REG, ACK);
        // follow with writing 7 bytes to registers 0 to 6
        for(i=0; i<7; i++){
            I2C_Write(regs[reg_translate[i]], ACK);
        }
        I2C_Write(CONTROL_INIT_VAL, ACK);     // DS1307 control register cfg
        I2C_Stop();

        printf("OK");
        DS1307_read_time(1);
    }else{
        printf("No data");
    }
}



// read time & date from DS1307
void read_time(void)
{
	I2C_Start();
	I2C_Write(WRITE_CNTRL, ACK);
	// start i2c read at seconds register
	I2C_Write(SECONDS_REG, ACK);
	I2C_Start();
	I2C_Write(READ_CNTRL, ACK);
	// read the 7 bytes from the ds1307. Mask off the unused bits
	ds1307_regs[SECONDS_REG]     = I2C_Read(ACK) & 0x7f;
	ds1307_regs[MINUTES_REG]     = I2C_Read(ACK) & 0x7f;
	ds1307_regs[HOURS_REG]       = I2C_Read(ACK) & 0x3f;
	ds1307_regs[DAY_OF_WEEK_REG] = I2C_Read(ACK) & 0x07;
	ds1307_regs[DATE_REG]        = I2C_Read(ACK) & 0x3f;
	ds1307_regs[MONTH_REG]       = I2C_Read(ACK) & 0x1f;
	ds1307_regs[YEAR_REG]        = I2C_Read(NOACK);

	I2C_Stop();

}

void DS1307_read_time(unsigned char select)
{

unsigned char buf[20];

    read_time();                  // read & display DS1307 time

    sprintf(buf, "%02X:%02X:%02X %02X-%02X-%02X ",
            (unsigned int) ds1307_regs[HOURS_REG],
            (unsigned int) ds1307_regs[MINUTES_REG],
            (unsigned int) ds1307_regs[SECONDS_REG],
            (unsigned int) ds1307_regs[YEAR_REG],
            (unsigned int) ds1307_regs[MONTH_REG],
            (unsigned int) ds1307_regs[DATE_REG]
            );

	if(0 == select){	
		set_LCD_cur(0, 0);
		puts_I2C_LCD(buf);
	}else{
		puts(buf);
	}
}


#define DS1307_NVRAM_START_ADR 		7
#define DS1307_NVRAM_END 			0x3F


void DS1307_REG_DUMP(void)
{

    unsigned char i;
    unsigned char j=0;

    printf("\r\nDS1307 RAM dump:\r\n");

    I2C_Start();
    I2C_Write(WRITE_CNTRL, ACK);

    // start i2c read at seconds register
    I2C_Write(SECONDS_REG, ACK);

    I2C_Start();
    I2C_Write(READ_CNTRL, ACK);

    for(i=0;i<DS1307_NVRAM_END;i++)
    {
        printf("%02X ", (unsigned int) I2C_Read(ACK));
        if(++j == 16)
        {
            j=0;
            printf("\r\n");
        }
    }
    printf("%02X ", (unsigned int)I2C_Read(NOACK));
    I2C_Stop();
}



// Clear all User RAM without affecting the first eight RTC data registers
void DS1307_Clear_all_RAM(void)
{
    unsigned char i = (unsigned char) DS1307_NVRAM_END - (unsigned char) DS1307_NVRAM_START_ADR;

    I2C_Start();
    I2C_Write(WRITE_CNTRL, ACK);              // DS1307 Slave Address
    //start at beginning of user RAM)
    I2C_Write(DS1307_NVRAM_START_ADR + 1, ACK);
    // follow with writing n bytes to registers
    while(i--)
    {
        I2C_Write(0, ACK);
    }
    I2C_Stop();
}


// LCD support

/* LCD WIRING for 120023G-LCD-R0 board driven from PORT0 of Mikroe 8051 Readyboard:
*
*	J2:		FUNCTION		PORT0 	NAME	COMMENT
*	-----------------------------------------------------------------------
*	1		SDA				7 		P0.7	I2C is software based
*	2		SCL				8		P0.6
*	10		GND				10		GND
*	9		+5V				9		VCC	
*
*   PIN     FUNCTION            PCA9554		DESCRIPTION
*	-----------------------------------------------------------------------
*   1       VSS                 			GND
*   2       VCC                 			+5V
*   3       VO                  			10K POT VCC to GND  Contrast Voltage
*   4       RS                  IO-0_2		
*   5       WR                  IO-0_1		
*   6       E               	IO-0_0		Automatic strobe generator with U3
*   7-10    DB0-3 (not used)
*   11      DB4                 IO-0_4		P4
*   12      DB5                 IO-0_5		P5
*   13      DB6                 IO-0_6		P6
*   14      DB7                 IO-0_7		P7
*	15		BL+					IO-0_3		Backlight Enable (0=ON) (Q1)
*	16		BL-
*/


unsigned char _lcd_addr=0, _lcd_line=0;
//00h, 40h, 14h, 54h  OR with | 0x80:
unsigned char addLUT[4] = {0x80, 0xC0, 0x94, 0xD4};

bit _HD44780_STROBE = 0;

#define LCD_CMD  0x00
#define LCD_DATA 0x02 // P1
#define LCD_WRITE 0x4E



#define STROBE_BIT 0x01




void wr_disp(unsigned char LCD_data, unsigned char  cmd_mode)
{
	unsigned char temp;
    // 
	temp = LCD_data;

    LCD_data &= 0xF0;	
	LCD_data |= cmd_mode << 1;	// Controls RS bit (P0_1: 0=CMD, 1=DATA)
	// send upper nibble
	write_PCA9554_LCD(LCD_data, PCA9554_OUTPUT_PORT, LCD_WRITE);
	
	LCD_data = temp & 0x0F;
	LCD_data = LCD_data << 4;		// Place 2nd half of byte into DB7-4 bit position
    LCD_data &= 0xF0;				// Clear RS and RW bits

	LCD_data |= cmd_mode << 1;	// Controls RS bit (P0_1: 0=CMD, 1=DATA)
	// send lower nibble
	write_PCA9554_LCD(LCD_data, PCA9554_OUTPUT_PORT, LCD_WRITE);
}





/******************************************************************************
* Function:     wr_half(unsigned char data)
* Description:  Write a nibble to the LCD display using DB4-DB7
* Returns:      none
*******************************************************************************/
void wr_half(unsigned char LCD_data, unsigned char cmd_mode)
{
	LCD_data = LCD_data << 4;	// Transpose bits to upper nibble
    LCD_data &= 0xF0;			// Clear RS and RW bits
    LCD_data |= cmd_mode << 1;	// Controls RS bit (P0_1: 0=CMD, 4=DATA)

    write_PCA9554_LCD(LCD_data, PCA9554_OUTPUT_PORT, LCD_WRITE);
}





void write_PCA9554_LCD(unsigned char dbyte, unsigned char cmd_byte, unsigned char slvadr)
{
	static unsigned char E = 0x01;

	// generate E strobe via U2 by toggling bit PCA9554 bit 0
	if(_HD44780_STROBE) dbyte &= ~E; else dbyte |= E;

	_HD44780_STROBE = ~_HD44780_STROBE;
	
	dbyte = dbyte & 0xF7;		// Clear BL bit
	
	I2C_Start();
	I2C_Write(slvadr, ACK);		// Enable Write mode
	I2C_Write(cmd_byte, ACK); 	// Use immediate transfer mode
	I2C_Write(dbyte, ACK);		// Write data
	I2C_Stop();
}



/******************************************************************************
* Function:     void disp_char(unsigned char c)
* Description:  display a character on LCD
* Uses:         character c, gl _lcd_line, _lcd_addr
* Returns:      none
*******************************************************************************/
void put_I2C_LCD(unsigned char c)
{
    wr_disp(c, LCD_DATA);

	_lcd_addr++;

	if(_lcd_addr == 20)
		line(1);
	else if(_lcd_addr == 40)
		line(2);
	else if(_lcd_addr == 60)
		line(3);
	else if(_lcd_addr == 80)
		line(0);

}



/******************************************************************************
* Function:     void disp_char(unsigned char c)
* Description:  display a string on LCD
* Uses:         character c, gl _lcd_line, _lcd_addr
* Returns:      none
*******************************************************************************/
void puts_I2C_LCD(unsigned char *c)
{
 
	while(*c)
	{
	   wr_disp(*c++, LCD_DATA);

		_lcd_addr++;
	
		if(_lcd_addr == 20)
			line(1);
		else if(_lcd_addr == 40)
			line(2);
		else if(_lcd_addr == 60)
			line(3);
		else if(_lcd_addr == 80)
			line(0);
	}
}



/******************************************************************************
* Function:     void line(char which_line)
* Description:  Move to line specified by which_line
* Uses:         which_line to select LCD line to be used
* Returns:      none
*******************************************************************************/
void line(unsigned char which_line)
{
    wr_disp(addLUT[which_line], LCD_CMD);                     // move to which_line line
    _lcd_line = which_line;
}



/******************************************************************************
* Function:     void clear_display(void)
* Description:  Clear LCD
* Uses:         none
* Returns:      none
*******************************************************************************/

void clear_display(void)
{
    // Clear display and home cursor
    wr_disp(0x01, LCD_CMD);
    delay_ms(2);
}



/******************************************************************************
* Function:     void set_LCD_cur(char LCDrow, char LCDcol)
* Description:  Set row, col cursor position in LCD
* Uses:         LCDrow, LCDcol
* Returns:      none
*******************************************************************************/

void set_LCD_cur(unsigned char LCDrow, unsigned char LCDcol)
{
   	_lcd_addr = addLUT[LCDrow];
	_lcd_addr += LCDcol;
   	_lcd_line = LCDrow;
   	wr_disp(_lcd_addr, LCD_CMD);          // move to row and column

}



/******************************************************************************
* Function:     void init_HD44780_LCD(void);
* Description:  Setup LCD display for application
* Returns:      none
*******************************************************************************/
void init_HD44780_LCD(void)
{
    // Provide power up delay for LCD controller

	delay_ms(100);

    // always command mode used here - for HD44780 LCD
    wr_half(0x03, LCD_CMD);     // Send 0x33 and 0x32 to enter 4 bit mode
    wr_half(0x03, LCD_CMD);     // See HD44780U.PDF datasheet for all
    wr_half(0x03, LCD_CMD);     // configuration options
    wr_half(0x02, LCD_CMD);

    wr_disp(0x28, LCD_CMD);  	// Mode: 4-Bit, 2-line, 5x7 - This is actually setting the mode
    wr_disp(0x01, LCD_CMD);  	// Clear Display memory
    delay_ms(20);            	// HD44780 requires 18ms for clear command to execute
    wr_disp(0x10, LCD_CMD);  	// Set cursor to move
    wr_disp(0x06, LCD_CMD);  	// set cursor to move right, no shifting of LCD display

    wr_disp(0x0c, LCD_CMD);  	// turn display on, cursor off, cursor blinking off
}



/******************************************************************************
* Function:
* Description:
* Returns:      none
*******************************************************************************/

void clear_line(unsigned char line)
{

	unsigned char i,num_chars;
	
	set_LCD_cur(line,0);

	num_chars = 40;

	for(i=0;i<num_chars;i++)
	{
		put_I2C_LCD(32);
	}
	
}



void write_PCA9554_port(unsigned char dbyte, unsigned char cmd_byte, unsigned char I2C_address)
{
	I2C_Start();
	I2C_Write(I2C_address, ACK);	// Enable Write mode
	I2C_Write(cmd_byte, ACK); 		// Use immediate transfer mode
	I2C_Write(dbyte, ACK);			// Write data
	I2C_Stop();
}



unsigned char read_PCA9554_port(unsigned char I2C_address)
{
	unsigned char regdata;
	
	I2C_Start();
	I2C_Write(I2C_address, ACK);				// Enable Write mode
	I2C_Write(PCA9554_INPUT_PORT, ACK); 		// command byte
	I2C_Start();
	I2C_Write(I2C_address | 0x01, ACK);			// Enable READ mode	
	regdata = I2C_Read(NOACK);					// Copy data from PCA9554 and terminate READ with NOACK
	I2C_Stop();

	return regdata;
}
//******************************************************************************************

/*

************* TABLE 1 ****************

BAUD RATES in X2 mode with BRG used:
f[crystal] = 16.384 MHZ

BR		BRLld	BRL-k	Ntot	SPD	PRE
--------------------------------------------------------
115200	247		9		144		1	1
57600	238		18		288		1	1
38400	229		27		432		1	1
19200	203		53		848		1	1
14400	185		71		1136	1	1
9600	149		107		1712	1	1
-------------------------------------
4800	 43		72		1152	0	6
2400	238		18		2304	0	6
1200	220		48		4608	0	6
600		185		96		9216	0	6

*/


void initialize_hardware(void)
{

//--------------------------------------------------------------------------
// Setup the serial port for x baud at 16 MHz using BRG.as per table 1
//--------------------------------------------------------------------------
    CKRL	= 0xFF;					// Clock divider disabled
	CKCON0 	= 0x7F;					// Set DIV BY 1
	PCON   |= 0x40;					// Enable FrameError detection
	PCON   |= 0x80;					// Double BRG clock rate
	SCON 	= 0x50; 				// 8-bit UART in mode 1, REN=1
	BDRCON &=0xEE; 					// Force BRR=0; SRC=0;
 	BDRCON |=0x0E; 					// Enable TBCK=1;RBCK=1; SPD=1
 	BRL		= 149;//220;		 	// See table 1 (16MHz -> 9600 = 149)
    TI      = 1;                  	// TI:   set TI to send first char of UART
	BDRCON |=0x10; 					// Baud rate generator start run
//--------------------------------------------------------------------------
// Setup the TIMER1 interrupt for 200 HZ or a 5ms rate for timebase purposes
//--------------------------------------------------------------------------
	CLKREG	= 0xB0;				// Peripheral Prescaler DIV=12 (K+1) K=11
	TR1 	= 0;
	RH1 	= T1_HIGH;
	RL1 	= T1_LOW;
	TMOD 	= TMOD | 0x10;          // TMOD: timer 1, mode 1, 16-bit ISR re-load
	TR1 	= 1;                    // start timer 1
//--------------------------------------------------------------------------
// ADC SETUP
//--------------------------------------------------------------------------
	DADI	=	0;
	DADI	|=  (1 << 7);	 						// ACON = ENABLED
	DADI	|=  (0 << 6); 							// IREF = 1V.0 REF / VDD/2
	DADI	|=  (1 << 5) | ( 0 << 4);				// TRG1/0 = TIMER1 O.F.
	DADC	|=	(1 << 4) | (1 << 3); 				// ADCE=1  (DADC Enable)
//	DADI	|=  (1 << 3);  							// DIFF = DIFF = ON
	DADI	|=  (0 << 2) | ( 1 << 1) | ( 1 << 0);	// Vtemp/VDD/8 SELECTED
//--------------------------------------------------------------------------
// ISR SETUP
//--------------------------------------------------------------------------

	IEN1 |= 0x20;						// Enable ADC interrupt
	ET1 	= 1;                    	// enable timer 1 interrupt
	ES 		= 1;						// Enable serial interrupt

	EA = 1;       						// global interrupt enable
// -------------------------------------------------------------------------
// Watchdog time-out setup
// -------------------------------------------------------------------------
	// For mode X1: 12 clock periods per machine cycle
	// For mode X2:  6 clock periods per machine cycle
#ifdef WATCHDOG_ON
	WDTPRG |= 0x07; 	// (2^(14+0x07)-1) machine cycles 
						//   [0x07=MAX] or 0.419s @ Fosc=30MHz & X2
#endif

// -------------------------------------------------------------------------
	memset(_ioregs, 0, MAXREGS);	// Clear XREGS

//	Initialize external devices

	// Clear LED board adapter
	TPIC2810(0x00);
	
	// Configure LED driver board
	write_PCA9554_port(255, PCA9554_OUTPUT_PORT, 0x4A);
	write_PCA9554_port(0, PCA9554_CFG_REG_PORT, 0x4A);

	// Configure LCD driver board
	write_PCA9554_port(0, PCA9554_CFG_REG_PORT, LCD_WRITE);
	write_PCA9554_port(0, PCA9554_OUTPUT_PORT, LCD_WRITE);

	init_HD44780_LCD();		// Configure HD44780 regs

	read_cal_values();		// Copy EEEPROM operational data on startup
	write_cal_values();		// Initializes the EEPROM with default data when sign. byte is wrong
	write_TMP101();			// Configure TMP101 regs

#ifdef STARTUP_MESSAGE_ENABLED
	printf("\r\nPCM3 1 STARTUP OK\n");
#endif
}



void main(void)
{
	initialize_hardware();

	PIN_P1_1=0;

	for(;;)
	{
		check_cmd_buffer();
		update_IO();
		update_RTC();
		restart_wdg();

		// Scope port timing test only with ALE (pin-30)
		PIN_P1_1=1;
		PIN_P1_1=0;

	} 
}
